@@ -14,6 +14,7 @@ from acasclient import acasclient
 from acasclient.ddict import ACASDDict, ACASLsThingDDict
 from acasclient.lsthing import (BlobValue, CodeValue, FileValue, LsThingValue,
                                 SimpleLsThing, get_lsKind_to_lsvalue)
+from acasclient.validation import ValidationResult, get_validation_response
 
 # SETUP
 # "bob" user name registered
@@ -117,6 +118,12 @@ class TestLsThing(unittest.TestCase):
         new_bytes = self._get_bytes(file_path)
         orig_bytes = self._get_bytes(orig_file_path)
         self.assertEqual(new_bytes, orig_bytes)
+    
+    def _test_codevalue_missing_error(self, message, value, code_type, code_kind, code_origin):
+        base_msg = "'{code}' is not yet in the database as a valid '{code_kind}'. Please double-check the spelling and correct your data if you expect this to match an existing term. If this is a novel valid term, please contact your administrator to add it to the following dictionary: Code Type: {code_type}, Code Kind: {code_kind}, Code Origin: {code_origin}"
+        expected_msg = base_msg.format(code=value, code_type=code_type, code_kind=code_kind, code_origin=code_origin)
+        self.assertEqual(message, expected_msg)
+
 
     # Tests
     def test_000_simple_ls_thing_save(self):
@@ -350,7 +357,8 @@ class TestLsThing(unittest.TestCase):
 
         # Test updating other values on a saved Thing
         saved_project = Project.get_by_code(newProject.code_name, self.client, Project.ls_type, Project.ls_kind)
-        saved_project.metadata[PROJECT_METADATA][STATUS_KEY] = 'inactive'
+        STATUS_DDICT = ACASDDict(PROJECT, STATUS)
+        saved_project.metadata[PROJECT_METADATA][PROJECT_STATUS] = CodeValue(INACTIVE, ddict=STATUS_DDICT)
         saved_project.save(self.client)
 
         # Test updating FileValue on a saved Thing
@@ -738,7 +746,7 @@ class TestLsThing(unittest.TestCase):
         assert not valid
         messages = valid.get_messages()
         assert len(messages) == 1
-        self.assertEqual(messages[0], f"Invalid 'code':'{status_1}' provided for the given 'code_type':'{PROJECT}' and 'code_kind':'{STATUS}'")
+        self._test_codevalue_missing_error(messages[0], status_1, PROJECT, STATUS, 'ACAS DDict')
         # Now test timing of one-by-one validation with 20 projects versus doing it in bulk
         # Create 20 valid projects
         meta_dict[STATUS_KEY] = ACTIVE
@@ -774,21 +782,21 @@ class TestLsThing(unittest.TestCase):
         proj_1 = Project(recorded_by=self.client.username, **meta_dict)
         # set status to a CodeValue constructed with a DDict
         STATUS_DDICT = ACASDDict(PROJECT, STATUS)
-        proj_1.metadata[PROJECT_METADATA][STATUS_KEY] = CodeValue(ACTIVE, ddict=STATUS_DDICT)
+        proj_1.metadata[PROJECT_METADATA][PROJECT_STATUS] = CodeValue(ACTIVE, ddict=STATUS_DDICT)
         # Because we are referencing a valid status, this should not raise an error
         valid = proj_1.validate(self.client)
         assert valid
         assert len(valid.get_messages()) == 0
         # Now try setting status to an invalid CodeValue
         status_1 = str(uuid.uuid4())
-        proj_1.metadata[PROJECT_METADATA][STATUS_KEY] = CodeValue(status_1, ddict=STATUS_DDICT)
+        proj_1.metadata[PROJECT_METADATA][PROJECT_STATUS] = CodeValue(status_1, ddict=STATUS_DDICT)
         valid = proj_1.validate(self.client)
         assert not valid
         assert len(valid.get_messages()) == 1
-        self.assertEqual(valid.get_messages()[0], f"Invalid 'code':'{status_1}' provided for the given 'code_type':'{PROJECT}' and 'code_kind':'{STATUS}'")
+        self._test_codevalue_missing_error(valid.get_messages()[0], status_1, PROJECT, STATUS, 'ACAS DDict')
         # Now try adding a CodeValue that references an LsThing
         # First we create and save a new LsThing so we can get a code_name
-        proj_1.metadata[PROJECT_METADATA][STATUS_KEY] = CodeValue(ACTIVE, ddict=STATUS_DDICT)
+        proj_1.metadata[PROJECT_METADATA][PROJECT_STATUS] = CodeValue(ACTIVE, ddict=STATUS_DDICT)
         proj_1.save(self.client)
         # Then we create a new project 2 and set PARENT_PROJECT_KEY to reference `proj_1`
         name_2 = str(uuid.uuid4())
@@ -810,7 +818,22 @@ class TestLsThing(unittest.TestCase):
         proj_2.metadata[PROJECT_METADATA][PARENT_PROJECT_KEY] = CodeValue(bad_project_code, ddict=PARENT_PROJECT_DDICT)
         valid = proj_2.validate(self.client)
         assert not valid
-        self.assertEqual(valid.get_messages()[0], f"Invalid 'code':'{bad_project_code}' provided for the given 'code_type':'{PROJECT}' and 'code_kind':'{PROJECT}'")
+        self._test_codevalue_missing_error(valid.get_messages()[0], bad_project_code, PROJECT, PROJECT, 'ACAS LsThing')
+        # Generate validation response and check it
+        validation_response = get_validation_response(valid, ls_thing=proj_2)
+        # Confirm the structure of the response body
+        assert validation_response.get("commit") is False
+        self.assertEqual(validation_response.get("transaction_id"), -1)
+        assert validation_response.get("hasError") is True
+        assert validation_response.get("hasWarning") is False
+        assert validation_response.get("results") is not None
+        assert validation_response.get("results").get("thing") is not None
+        # Check we have one message and it is an error
+        self.assertEqual(len(validation_response.get("errorMessages")), 1)
+        msg = validation_response.get("errorMessages")[0]
+        self.assertEqual(msg.get("errorLevel"), "error")
+        # Do a loose check of the html summary and confirm it contains mention of our bad code
+        assert bad_project_code in validation_response['results']['htmlSummary']
 
 
 class TestBlobValue(unittest.TestCase):
@@ -836,3 +859,73 @@ class TestBlobValue(unittest.TestCase):
         assert blob_value_dict['value'] == value
         assert blob_value_dict['comments'] == comments
         assert blob_value_dict['id'] == id
+
+class TestValidationResponse(unittest.TestCase):
+
+    def setUp(self) -> None:
+        creds = acasclient.get_default_credentials()
+        self.client = acasclient.client(creds)
+
+    def tearDown(self):
+        """Tear down test fixtures, if any."""
+        self.client.close()
+
+    def test_001_response_with_errors_and_warnings(self):
+        """
+        Test creating a ValidationResponse with errors and warnings.
+        """
+        ERR_MSG = 'error 1'
+        WARN_MSG = 'warning 1'
+        # Create a warning
+        warn = ValidationResult(True, [WARN_MSG])
+        assert warn
+        assert len(warn.get_messages()) == 1
+        assert len(warn.get_warnings()) == 1
+        assert warn.get_warnings()[0] == WARN_MSG
+        assert len(warn.get_errors()) == 0
+
+        # Create an error
+        err = ValidationResult(False, [ERR_MSG])
+        assert not err
+        assert len(err.get_messages()) == 1
+        assert len(err.get_warnings()) == 0
+        assert len(err.get_errors()) == 1
+        assert err.get_errors()[0] == ERR_MSG
+
+        # Add them
+        valid = warn + err
+        assert not valid
+        assert len(valid.get_messages()) == 2
+        assert valid.get_messages()[0] == ERR_MSG
+        assert valid.get_messages()[1] == WARN_MSG
+        assert len(valid.get_errors()) == 1
+        assert valid.get_errors()[0] == ERR_MSG
+        assert len(valid.get_warnings()) == 1
+        assert valid.get_warnings()[0] == WARN_MSG
+
+        # Generate and check a response
+        response = get_validation_response(valid)
+        # Confirm the structure of the response body
+        assert response.get("commit") is False
+        self.assertEqual(response.get("transaction_id"), -1)
+        assert response.get("hasError") is True
+        assert response.get("hasWarning") is True
+        assert response.get("results") is not None
+        # Check we have two messages: one error, one warning
+        self.assertEqual(len(response.get("errorMessages")), 2)
+        self.assertEqual(response.get("errorMessages")[0].get("errorLevel"), "error")
+        self.assertEqual(response.get("errorMessages")[1].get("errorLevel"), "warning")
+        # Confirm HTML summary contains both messages
+        assert ERR_MSG in response['results']['htmlSummary']
+        assert WARN_MSG in response['results']['htmlSummary']
+
+        # Create a ValidationResult with error and warning at once
+        valid = ValidationResult(False, errors=[ERR_MSG], warnings=[WARN_MSG])
+        assert len(valid.get_messages()) == 2
+
+        # Create an invalid ValidationResult by giving an error message but saying it is valid
+        try:
+            invalid = ValidationResult(True, errors=[ERR_MSG], warnings=[WARN_MSG])
+            assert False
+        except ValueError as e:
+            assert str(e) == "ValidationResult cannot be valid and contain error messages"
